/*******************************************************************************
*  skMPL115A1 - 大気圧センサMPL115A1(SPI)用関数ライブラリ                      *
*                                                                              *
*    skMPL115A1      - MPL115A1関数ライブラリを生成する時の初期化処理          *
*    CoefficientRead - メモリーマップから係数を読み出す処理                    *
*    PressureRead    - メモリーマップから圧力と温度のA/D変換値を読み出す処理   *
*    PressureAVE     - 圧力･温度のA/D変換値を平均化させ気圧値を計算する処理    *
*    PressureCalc    - 気圧値(hPa)を計算する処理                               *
*    AltitudeCalc    - 気圧値(hPa)から高度を計算する処理                       *
*                                                                              *
* ============================================================================ *
*   VERSION  DATE        BY             CHANGE/COMMENT                         *
* ---------------------------------------------------------------------------- *
*   1.00     2012-12-27  きむ茶工房     Create                                 *
* ============================================================================ *
*   Arduino IDE 1.0.1 (Ardino Duemilanove 328)                                 *
*******************************************************************************/
#include <SPI.h>
#include "pins_arduino.h"
#include "arduino.h"
#include "skMPL115A1.h"


float skMPL115A1::a0 ;                  // ａ０係数のデータを保存する変数
float skMPL115A1::b1 ;                  // ｂ１係数のデータを保存する変数
float skMPL115A1::b2 ;                  // ｂ２係数のデータを保存する変数
float skMPL115A1::c12 ;                 // ｃ１２係数のデータを保存する変数
unsigned long skMPL115A1::Press ;       // 圧力の変換値を保存する変数
unsigned long skMPL115A1::Temp ;        // 温度の変換値を保存する変数

/*******************************************************************************
*  skMPL115A1(divider,mode)                                                    *
*    MPL115A1関数ライブラリを生成する時の初期化処理(コンストラクタ)            *
*    divider : SPIクロック分周器を設定します。(SPI_CLOCK_DIV2/4/8/16/32/64/128)*
*    mode    : SPIの転送モードを設定します。(SPI_MODE0/1/2/3)                  *
*******************************************************************************/
skMPL115A1::skMPL115A1(int divider,int mode)
{
     SPI.begin() ;                      // ＳＰＩを行う為の初期化
     SPI.setBitOrder(MSBFIRST) ;        // ビットオーダー
     SPI.setClockDivider(divider) ;     // ＳＰＩのクロック
     SPI.setDataMode(mode) ;            // クロック極性とクロック位相
}
/*******************************************************************************
*  CoefficientRead()                                                           *
*    メモリーマップから係数を読み出す処理                                      *
*******************************************************************************/
void skMPL115A1::CoefficientRead()
{
     unsigned int h , l ;

     digitalWrite(SS,LOW) ;               // SS(CS)ラインをLOWにする

     // ａ０の係数を得る
     SPI.transfer(0x88) ;                 // a0(MSB:HIGH byte)係数
     h = SPI.transfer(0x00) ;
     SPI.transfer(0x8a) ;                 // a0(LSB:LOW byte) 係数
     l = SPI.transfer(0x00) ;
     a0 = (h << 5) + (l >> 3) + (l & 0x07) / 8.0 ;

     // ｂ１の係数を得る
     SPI.transfer(0x8c) ;                 // b1(MSB:HIGH byte)係数
     h = SPI.transfer(0x00) ;
     SPI.transfer(0x8e) ;                 // b1(LSB:LOW byte) 係数
     l = SPI.transfer(0x00) ;
     b1 = ( ( ( (h & 0x1F) * 0x100 ) + l ) / 8192.0 ) - 3 ;

     // ｂ２の係数を得る
     SPI.transfer(0x90) ;                 // b2(MSB:HIGH byte)係数
     h = SPI.transfer(0x00) ;
     SPI.transfer(0x92) ;                 // b2(LSB:LOW byte) 係数
     l = SPI.transfer(0x00) ;
     b2 = ( ( ( ( h - 0x80) << 8 ) + l ) / 16384.0 ) - 2 ;

     // Ｃ１２の係数を得る
     SPI.transfer(0x94) ;                 // c12(MSB:HIGH byte)係数
     h = SPI.transfer(0x00) ;
     SPI.transfer(0x96) ;                 // c12(LSB:LOW byte) 係数
     l = SPI.transfer(0x00) ;
     c12 = ( ( ( h * 0x100 ) + l ) / 16777216.0 )  ;

     SPI.transfer(0x00) ;
     digitalWrite(SS,HIGH) ;              // SS(CS)ラインをHIGHにする
}
/*******************************************************************************
*  PressureRead()                                                              *
*    メモリーマップから圧力および温度のＡ／Ｄ変換値を読み出す処理              *
*******************************************************************************/
void skMPL115A1::PressureRead()
{
     unsigned int h , l ;

     // 圧力および温度の変換を開始させる
     digitalWrite(SS,LOW) ;               // SS(CS)ラインをLOWにする
     SPI.transfer(0x24) ;                 // 0x24コマンドの発行(圧力と温度の変換)
     SPI.transfer(0x00) ;
     digitalWrite(SS,HIGH) ;              // SS(CS)ラインをHIGHにする
     delay(3) ;                           // ３ｍｓ待つ

     digitalWrite(SS,LOW) ;               // SS(CS)ラインをLOWにする

     // 圧力のＡ／Ｄ変換値を得る
     SPI.transfer(0x80) ;                 // 圧力(MSB:HIGH byte)
     h = SPI.transfer(0x00) ;
     SPI.transfer(0x82) ;                 // 圧力(LSB:LOW byte)
     l = SPI.transfer(0x00) ;
     Press = ( ( h * 256 ) + l ) / 64 ;

     // 温度のＡ／Ｄ変換値を得る
     digitalWrite(SS,LOW) ;               // SS(CS)ラインをLOWにする
     SPI.transfer(0x84) ;                 // 温度(MSB:HIGH byte)
     h = SPI.transfer(0x00) ;
     SPI.transfer(0x86) ;                 // 温度(LSB:LOW byte)
     l = SPI.transfer(0x00) ;
     Temp = ( ( h * 256 ) + l ) / 64 ;

     SPI.transfer(0x00) ;
     digitalWrite(SS,HIGH) ;              // SS(CS)ラインをHIGHにする
}
/*******************************************************************************
*  ans = PressureAVE(count)                                                    *
*    圧力･温度のA/D変換値を平均化させ気圧値を計算する処理                      *
*    count : A/D変換値を平均の為に何回読み込むか指定する。                     *
*    ans   : 大気圧値を hPa で返す。                                           *
*******************************************************************************/
float skMPL115A1::PressureAVE(int count)
{
     int i ;
     float ans ;
     unsigned long p , t ;

     p = t = 0 ;
     for (i=0 ; i < count ; i++) {      // 指定回数ぶん読み込む
          PressureRead() ;              // メモリーマップから圧力と温度のA/D変換値を読み出す
          p = p + Press ;
          t = t + Temp ;
     }
     Press = p / count ;                // 単純平均化を行う
     Temp  = t / count ;
     ans = PressureCalc() ;             // 気圧値の計算を行う

     return ans ;
}
/*******************************************************************************
*  ans = PressureCalc()                                                        *
*    気圧値(hPa)を計算する処理                                                 *
*    ans : 大気圧値を hPa で返す。                                             *
*******************************************************************************/
float skMPL115A1::PressureCalc()
{
     float ret , f ;

     f = a0 + ( b1 + c12 * Temp ) * Press + b2 * Temp ;
     ret = f * ( 650.0 / 1023.0 ) + 500.0 ;
     return ret ;
}
/*******************************************************************************
*  ans = AltitudeCalc(pressure,Difference)                                     *
*    気圧値(hPa)から高度を計算する処理                                         *
*    pressure   : 計算した大気圧値を hPa でセットする。                        *
*    Difference : 標高の差を指定する                                           *
*    ans        : 高度値を ｍ で返す。                                         *
*******************************************************************************/
float skMPL115A1::AltitudeCalc(float pressure,int Difference)
{
     float h ;

     h = 44330.8 * (1.0 - pow( (pressure/1013.25) ,  0.190263 )) ;
     h = h + Difference ;
     return h ;
}
